#ifdef __WIN32__
#include <winsock.h>
#else
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#endif
#include "stdio.h"
#include "stdlib.h"
#include "string.h"

#define MAX_PACKET 1024

#define ICMP_MINLEN 		8
#define ICMP_ECHOREPLY		0	/* Echo Reply			*/
#define ICMP_DEST_UNREACH	3	/* Destination Unreachable	*/
#define ICMP_SOURCE_QUENCH	4	/* Source Quench		*/
#define ICMP_REDIRECT		5	/* Redirect (change route)	*/
#define ICMP_ECHO		8	/* Echo Request			*/
#define ICMP_TIME_EXCEEDED	11	/* Time Exceeded		*/
#define ICMP_PARAMETERPROB	12	/* Parameter Problem		*/
#define ICMP_TIMESTAMP		13	/* Timestamp Request		*/
#define ICMP_TIMESTAMPREPLY	14	/* Timestamp Reply		*/
#define ICMP_INFO_REQUEST	15	/* Information Request		*/
#define ICMP_INFO_REPLY		16      /* Information Reply		*/
#define ICMP_ADDRESS		17	/* Address Mask Request		*/
#define ICMP_ADDRESSREPLY	18	/* Address Mask Reply		*/
#define NR_ICMP_TYPES		18

/* Codes for UNREACH. */
#define ICMP_NET_UNREACH	0	/* Network Unreachable		*/
#define ICMP_HOST_UNREACH	1	/* Host Unreachable		*/
#define ICMP_PROT_UNREACH	2	/* Protocol Unreachable		*/
#define ICMP_PORT_UNREACH	3	/* Port Unreachable		*/
#define ICMP_FRAG_NEEDED	4	/* Fragmentation Needed/DF set	*/
#define ICMP_SR_FAILED		5	/* Source Route failed		*/
#define ICMP_NET_UNKNOWN	6
#define ICMP_HOST_UNKNOWN	7
#define ICMP_HOST_ISOLATED	8
#define ICMP_NET_ANO		9
#define ICMP_HOST_ANO		10
#define ICMP_NET_UNR_TOS	11
#define ICMP_HOST_UNR_TOS	12
#define ICMP_PKT_FILTERED	13	/* Packet filtered */
#define ICMP_PREC_VIOLATION	14	/* Precedence violation */
#define ICMP_PREC_CUTOFF	15	/* Precedence cut off */
#define NR_ICMP_UNREACH		15      /* instead of hardcoding immediate value */

char ver_fp[] = "Fast ping Version 0.1\n";

typedef struct icmp {
  char i_type;
  char i_code;
  short i_cksum;
  short i_id;
  short i_seq;
  char data[30];
}IcmpHeader;

struct ip
  {
    char ip_ver;
    char ip_tos;
    short  ip_len;
    short  ip_id;
    short  ip_off;
    char ip_ttl;
    char ip_p;
    short  ip_sum;
    long ip_src,ip_dst;
  };

 fd_set fds;
 struct timeval wait;
 int sock_raw,datasize,hlen;
 long timeout;
 struct sockaddr_in dest;
 int in_size = sizeof(dest);
 char *recvbuf;
 char *icmp_data;
 char *icmp;
 struct hostent *hp;
 struct ip *ip;

int fp(long *dest_ip, int ttl, long timeout)
{


#ifdef __WIN32__
	 WORD        VersionRequested;
	 WSADATA     ImplementationDetails;
	 VersionRequested = MAKEWORD(1,1);
	 if ( WSAStartup (VersionRequested, &ImplementationDetails) ) {	return(-1);};
#endif

 icmp_data = malloc(MAX_PACKET);
 recvbuf = malloc(MAX_PACKET);
 dest.sin_family = AF_INET;
 dest.sin_addr.s_addr = *dest_ip;
 ((IcmpHeader*)icmp_data)->i_type = ICMP_ECHO;
 ((IcmpHeader*)icmp_data)->i_code = 0;
 ((IcmpHeader*)icmp_data)->i_cksum = 0;
 ((IcmpHeader*)icmp_data)->i_id = htons(0x0001);
 ((IcmpHeader*)icmp_data)->i_seq = htons(0x0002);
 strcpy(((IcmpHeader*)icmp_data)->data,ver_fp);
 ((IcmpHeader*)icmp_data)->i_cksum = checksum((short *)icmp_data,sizeof(struct icmp));
if((sock_raw=socket(AF_INET,SOCK_RAW,IPPROTO_ICMP))==-1)
 {perror("socket()");return(1);}
if(setsockopt(sock_raw,IPPROTO_IP,IP_TTL,(char *)&ttl,sizeof(ttl))==-1)
 {perror("setsockopt(IP_TTL)");return(-1);}
if(sendto(sock_raw,icmp_data,sizeof(struct icmp),0,(struct sockaddr *)&dest,in_size)<0)
 {perror("send()");return(1);}
FD_ZERO(&fds);
FD_SET(sock_raw, &fds);
wait.tv_sec = 0;
wait.tv_usec = timeout;
datasize = 0;
if (select(sock_raw+1, &fds, (fd_set *)0, (fd_set *)0, &wait) > 0)
datasize=recvfrom(sock_raw,recvbuf,MAX_PACKET,0,(struct sockaddr *)&dest,&in_size);
if(!datasize)return(4);
if(datasize==-1){perror("recv()");return(1);};
ip = (struct ip *) recvbuf;
hlen = ((ip->ip_ver)&0x0F)<<2;
memcpy(dest_ip,&ip->ip_src,sizeof(long));
if ( datasize < hlen+ICMP_MINLEN )return(3);
icmp = recvbuf + hlen;
if(((IcmpHeader*)icmp)->i_type==ICMP_DEST_UNREACH)return(1);
if(((IcmpHeader*)icmp)->i_type==ICMP_TIME_EXCEEDED)return(2);
if(((IcmpHeader*)icmp)->i_type==ICMP_ECHOREPLY)return(0);
fprintf(stdout,"%s return ICMP code:%d\n",inet_ntoa(dest.sin_addr),((IcmpHeader*)icmp)->i_type);
#ifdef __WIN32__
if((closesocket(sock_raw)==-1))
#else
if((close(sock_raw)==-1))
#endif
  {perror("closesocket()");return(-1);};
return(-1);
}

#ifdef FAST_PING
int main(int argc,char* argv[])
{
 long ip;
 if(argc!=4)
 {fprintf(stdout,"usage %s dest ttl timeout\n",argv[0]);return(-1);};
  fprintf(stdout,ver_fp);
  ip = inet_addr(argv[1]);
 switch (fp(&ip,atoi(argv[2]),atol(argv[3])))
        {
            case 0:
		fprintf(stdout,"Target is alive");	
                break;
            case 1:
		fprintf(stdout,"Destination unreachable");	
		break;
            case 2:
		fprintf(stdout,"Time exceeded");	
                break;
            case 3:
		fprintf(stdout,"Packet too short");	
                break;
            case 4:
		fprintf(stdout,"Time out");	
		break;
            default :
            fprintf(stdout,"Unknow anser\n");	
                break;
        }
  fprintf(stdout," from %s\n",ltoip(ip));
}
#endif